package org.scripps.eutils;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import gov.nih.nlm.uts.webservice.content.*;
import gov.nih.nlm.uts.webservice.security.*;
import gov.nih.nlm.uts.webservice.security.UtsFault_Exception;
import gov.nih.nlm.uts.webservice.metadata.*;
import gov.nih.nlm.uts.webservice.finder.*;
import gov.nih.nlm.uts.webservice.history.*;
import gov.nih.nlm.uts.webservice.semnet.*;

/**
 * Client for services described here: https://uts.nlm.nih.gov/home.html#apidocumentation
 * @author bgood
 *
 */
public class UmlsClient {

	//create the reference variables
	private UtsWsSemanticNetworkController utsSemanticNetworkService;
	private UtsWsSecurityController utsSecurityService;
	UtsWsContentController utsContentService;
	String username; 
	String password; 
	String umlsRelease;
	String serviceName = "http://umlsks.nlm.nih.gov";
	String ticketGrantingTicket = "";
	String isaRid = "T186";
	
	public UmlsClient(String username, String password, String umlsRelease){
		username = username;
		password = password;
		umlsRelease = umlsRelease;
		utsContentService = (new UtsWsContentControllerImplService()).getUtsWsContentControllerImplPort();
		utsSecurityService = (new UtsWsSecurityControllerImplService()).getUtsWsSecurityControllerImplPort();
		//get the Proxy Grant Ticket - this is good for 8 hours and is needed to generate single use tickets.
		try {
			ticketGrantingTicket = utsSecurityService.getProxyGrantTicket(username, password);
		} catch (UtsFault_Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//set up security service
		utsSecurityService = (new UtsWsSecurityControllerImplService())
				.getUtsWsSecurityControllerImplPort();
		
		//set up semantic network service
		utsSemanticNetworkService = (new UtsWsSemanticNetworkControllerImplService())
				.getUtsWsSemanticNetworkControllerImplPort();
	}
	
	public String getTicket() throws UtsFault_Exception{
		String ticket = utsSecurityService.getProxyTicket(ticketGrantingTicket, serviceName);
		return ticket;
	}
	
	
	public static void main(String[] args) throws gov.nih.nlm.uts.webservice.semnet.UtsFault_Exception, UtsFault_Exception {
		UmlsClient c = new UmlsClient("bgoodumls", "d1Hacker", "2013AA");
		String s1 = "T028";//"T121";// Pharmacological substance; //"T109"; //organic chemical  "T047"; //disease or disorder
		String s2 = "T047";//"T191"; //Neoplastic process "T028"; //gene or genome 
		
		c.getInheritedSemanticNetworkRelations(s1, s2);
		//c.getDirectSemanticNetworkRelations(s2, s1);
	}

	public Set<String> getParents(String stypeid, Set<String> parent_tuis) throws gov.nih.nlm.uts.webservice.semnet.UtsFault_Exception, UtsFault_Exception{
		if(parent_tuis==null){
			parent_tuis = new HashSet<String>(); 
		}
		List<SemanticTypeRelationDTO> relations = utsSemanticNetworkService.getInheritedSemanticTypeRelations(getTicket(), umlsRelease, stypeid);
		for (int i = 0; i < relations.size(); i++) {
			SemanticTypeRelationDTO mySemTyRel = relations.get(i);
			String relUi = mySemTyRel.getRelationLabel().getUi();
			if(relUi.equals(isaRid)){
				SemanticTypeDTO parent = mySemTyRel.getRelatedSemanticType();
				if(parent_tuis.add(parent.getUi())){
					parent_tuis.addAll(getParents(parent.getUi(), parent_tuis));
				}
				
			}
		}
		return parent_tuis;
	}
	

	public void getInheritedSemanticNetworkRelations(String stypeid1, String stypeid2){
		//instantiate and handshake
		try {
			
//			//get inherited semantic relations attached to type1
		List<SemanticTypeRelationDTO> relations = utsSemanticNetworkService.getInheritedSemanticTypeRelations(getTicket(), umlsRelease, stypeid1);
			//get parent types of type2
			Set<String> type2_parents = getParents(stypeid2, null);
			type2_parents.add(stypeid2);

			//alternate method of getting parent relations
			//not sure why it gets different results
//			Set<String> type1_parents = getParents(stypeid2, null);
//			type1_parents.add(stypeid1);
//			List<SemanticTypeRelationDTO> relations = new ArrayList<SemanticTypeRelationDTO>();
//			for(String tui1 : type1_parents){
//				relations.addAll(utsSemanticNetworkService.getInheritedSemanticTypeRelations(getTicket(), umlsRelease, tui1));
//			}
			
			//go through and report those that connect type1 to type2 or to type2's parents
			for (int i = 0; i < relations.size(); i++) {
				SemanticTypeRelationDTO mySemTyRel = relations.get(i);
				String relAbbr = mySemTyRel.getRelationLabel().getAbbreviation();
				String relLabel = mySemTyRel.getRelationLabel().getLabel();
				String relDef = mySemTyRel.getRelationLabel().getDefinition();
				String relInvLabel = mySemTyRel.getRelationLabel().getInverseLabel();
				String relTreeNum = mySemTyRel.getRelationLabel().getTreeNumber();
				String relUi = mySemTyRel.getRelationLabel().getUi();
				String relUsageNote = mySemTyRel.getRelationLabel().getUsageNote();
				int relChildCnt = mySemTyRel.getRelationLabel().getChildCount();
				int relRelCnt = mySemTyRel.getRelationLabel().getRelationCount();

				String SemTypeAbbr = mySemTyRel.getRelatedSemanticType().getAbbreviation();
				String SemTypeDef = mySemTyRel.getRelatedSemanticType().getDefinition();
				String SemTypeTreeNum = mySemTyRel.getRelatedSemanticType().getTreeNumber();
				String SemTypeUi = mySemTyRel.getRelatedSemanticType().getUi();
				String SemTypeVal = mySemTyRel.getRelatedSemanticType().getValue();
				String SemTypeUsage = mySemTyRel.getRelatedSemanticType().getUsageNote();
				int SemTypeChildCnt = mySemTyRel.getRelatedSemanticType().getChildCount();
				int SemTypeRelCnt = mySemTyRel.getRelatedSemanticType().getRelationCount();
				//System.out.println("-_"+SemTypeUi+"_-");
				if(type2_parents.contains(SemTypeUi)){
					System.out.println(relAbbr+"\t"+relLabel+"\t"+SemTypeVal+"\t"+SemTypeUi);
				}
			}

		}
		catch (Exception e) {
			System.out.println("Error!!!" + e.getMessage());
		}
	}
	
	
	public void getDirectSemanticNetworkRelations(String stypeid1, String stypeid2){
		//instantiate and handshake
		try {
			String ticket = utsSecurityService.getProxyTicket(ticketGrantingTicket, serviceName);
			List<SemanticTypeRelationDTO> myarrSemTyRelDTO = utsSemanticNetworkService.getSemanticTypeRelationsForPair (ticket, umlsRelease, stypeid1, stypeid2); //"T059" "T031"

			for (int i = 0; i < myarrSemTyRelDTO.size(); i++) {
				SemanticTypeRelationDTO mySemTyRel = myarrSemTyRelDTO.get(i);
				String relAbbr = mySemTyRel.getRelationLabel().getAbbreviation();
				String relLabel = mySemTyRel.getRelationLabel().getLabel();
				String relDef = mySemTyRel.getRelationLabel().getDefinition();
				String relInvLabel = mySemTyRel.getRelationLabel().getInverseLabel();
				String relTreeNum = mySemTyRel.getRelationLabel().getTreeNumber();
				String relUi = mySemTyRel.getRelationLabel().getUi();
				String relUsageNote = mySemTyRel.getRelationLabel().getUsageNote();
				int relChildCnt = mySemTyRel.getRelationLabel().getChildCount();
				int relRelCnt = mySemTyRel.getRelationLabel().getRelationCount();

				String SemTypeAbbr = mySemTyRel.getRelatedSemanticType().getAbbreviation();
				String SemTypeDef = mySemTyRel.getRelatedSemanticType().getDefinition();
				String SemTypeTreeNum = mySemTyRel.getRelatedSemanticType().getTreeNumber();
				String SemTypeUi = mySemTyRel.getRelatedSemanticType().getUi();
				String SemTypeVal = mySemTyRel.getRelatedSemanticType().getValue();
				String SemTypeUsage = mySemTyRel.getRelatedSemanticType().getUsageNote();
				int SemTypeChildCnt = mySemTyRel.getRelatedSemanticType().getChildCount();
				int SemTypeRelCnt = mySemTyRel.getRelatedSemanticType().getRelationCount();
				System.out.println(relAbbr+"\t"+"\t"+relLabel+"\t"+SemTypeVal);
			}

		}
		catch (Exception e) {
			System.out.println("Error!!!" + e.getMessage());
		}
	}
	
	public void getSemanticRelationsConceptConcept(){
		//login, get ticket
				try {
					//build some ConceptDTOs and retrieve UI and Default Preferred Name

					//use the Proxy Grant Ticket to get a Single Use Ticket
					String singleUseTicket1 = utsSecurityService.getProxyTicket(ticketGrantingTicket, serviceName);
					ConceptDTO result1 =  utsContentService.getConcept(singleUseTicket1, umlsRelease, "C0008487");

					System.out.println(result1.getUi() );
					System.out.println(result1.getDefaultPreferredName() );

					//use the Proxy Grant Ticket to get another Single Use Ticket
					String singleUseTicket2 = utsSecurityService.getProxyTicket(ticketGrantingTicket, serviceName);
					//            ConceptDTO result2 =  utsContentService.getConcept(singleUseTicket2, umlsRelease, "C0014591");
					//            System.out.println(result2.getUi() );
					//            System.out.println(result2.getDefaultPreferredName() );

					//check for relations
					gov.nih.nlm.uts.webservice.content.Psf myPsf = new gov.nih.nlm.uts.webservice.content.Psf();
					List<ConceptRelationDTO> myConceptRelationsDTO = new ArrayList<ConceptRelationDTO>();
					myConceptRelationsDTO = utsContentService.getConceptConceptRelations(singleUseTicket2, umlsRelease, "C0008487", myPsf);

					for (int i = 0; i < myConceptRelationsDTO.size(); i++) {

						ConceptRelationDTO myConceptRelationDTO = myConceptRelationsDTO.get(i);
						String otherConceptUi = myConceptRelationDTO.getRelatedConcept().getUi();
						String otherConceptName = myConceptRelationDTO.getRelatedConcept().getDefaultPreferredName();
						String otherConceptRel = myConceptRelationDTO.getRelationLabel();
						String otherConceptRela = myConceptRelationDTO.getAdditionalRelationLabel();
						System.out.println("chordoma rel:"+otherConceptRel+" otherconcept:"+otherConceptName);

					} 
				} catch (Exception ex) {
					ex.printStackTrace();
				}	

	}

}
