/**
 * 
 */
package org.scripps.samesame;

import gov.nih.nlm.ncbi.www.soap.eutils.EUtilsServiceStub;
import gov.nih.nlm.ncbi.www.soap.eutils.EUtilsServiceStub.DbInfoType;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.axis2.AxisFault;

/**
 * @author bgood
 *
 */
public class PubmedDistance {

	EUtilsServiceStub service;
	// call NCBI ESearch utility // NOTE: search term should be URL encoded 
	EUtilsServiceStub.ESearchRequest req1;
	double current_pubmed_size;
	Set<String> concept_q;
	Map<String, Double> concept_count;

	PubmedDistance() throws AxisFault{
		service = new EUtilsServiceStub();
		// call NCBI ESearch utility // NOTE: search term should be URL encoded 
		req1 = new EUtilsServiceStub.ESearchRequest();
		req1.setTool("personal_research");
		req1.setEmail("bgood@scripps.edu");
		req1.setDb("pubmed"); 
		setCurrentPubmedSize();
	}

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		PubmedDistance p = new PubmedDistance();
		String hitfile = "data/mesh_counts.txt";
		String meshtermfile = "data/cystinosis/top50_asu_filter_v2.txt";//"data/ngly1/top50_ngly1_PNGase_f"; //"data/ebola/top50bmgfilter_2"; //"data/cystinosis/top100_asu_filter.txt";
		String distance_matrix_file = "data/cystinosis/top50_asu_v2dist.txt";//"data/ngly1/top50dist.txt";
		String network_file = "data/cystinosis/top50_asu_v2_binarytree.txt"; //"data/ngly1/top50_bmg_binarytree.txt";
		//root should be equal to the original query.. hence it might not be a MeSH term..
		String root = "ctns OR cystinosis OR cysteamine";//"Ngly1 OR PNGase F";////"Hemorrhagic Fever, Ebola";//"Ebolavirus";		//"Cystinosis";
		p.allByAllMesh(hitfile, meshtermfile, distance_matrix_file, network_file, root, 100); //
	}

	public void allByAllMesh(String hitfile, String meshtermfile, String distance_matrix_file, String network_file, String root, int max_terms) throws IOException{
		//set pubmed size
		setCurrentPubmedSize();
		//load the term list of interest
		Set<String> terms = loadConcepts(meshtermfile, max_terms);
		//add the query in case it wasn't in MeSH
		terms.add(root);
		//load the cached counts (if any, else make)
		loadCachedHitCounts(hitfile);
		//gather all the counts (here with MeSH queries)
		for(String term : terms){
			//check if we have it
			if(!concept_count.keySet().contains(term)){
				try {
					//if not get it
					double c = getPubmedCount(term, "[MeSH Terms]");
					if(c==0){
						//this here mainly for case where term is not in MeSH
						//TODO clarify intent and update..
						c = getPubmedCount(term, "");
					}
					concept_count.put(term, c);
					save_search_result(term, c, hitfile);
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		//now get the intersections, calculate the distances, and write out the matrix
		FileWriter dmatrix = new FileWriter(distance_matrix_file);
		dmatrix.write(" \t");
		for(String term : terms){
			dmatrix.write(term+"\t");
		}
		dmatrix.write("\n");
		dmatrix.close();
		//keep the distances ready for network output
		Map<String, Double> concept_concept_distance = new HashMap<String, Double>();
		for(String source : terms){
			dmatrix = new FileWriter(distance_matrix_file, true);
			dmatrix.write(source+"\t");
			for(String target : terms){
				double dist = 0;
				String both = "("+source+" [MeSH Terms])"+" AND ("+target+" [MeSH Terms])";
				String reverse = "("+target+" [MeSH Terms])"+" AND ("+source+" [MeSH Terms])";
				if(!source.equals(target)){
					//check if we have it
					if(!concept_count.keySet().contains(both)){
						try {
							//again here handle the case where we have a non-mesh root/query term
							if(source.equals(root)){
								both = "("+source+")"+" AND ("+target+" [MeSH Terms])";
								reverse = "("+source+" [MeSH Terms])"+" AND ("+target+")";
							}else if(target.equals(root)){
								both = "("+source+" [MeSH Terms])"+" AND ("+target+")";
								reverse = "("+source+")"+" AND ("+target+" [MeSH Terms])";
							}

							double c = getPubmedCount(both, null);
							concept_count.put(both, c);				
							concept_count.put(reverse, c);
							save_search_result(both, c, hitfile);
							save_search_result(reverse, c, hitfile); //gag hack hack...
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
					dist = calculatePubmedDistance(concept_count.get(source), concept_count.get(target), concept_count.get(both));
					concept_concept_distance.put(source+"_"+target, dist);
				}
				dmatrix.write(dist+"\t");
			}
			dmatrix.write("\n");
			dmatrix.close();
		}
		//build the n-ary similarity tree
		//alter original code to set root at original query term..
		int max_children = 2;
		Set<String> entities = new HashSet<String>(terms);
		entities.remove(root);
		Map<String, Integer> root_candidates = new HashMap<String, Integer>();
		root_candidates.put(root, 0);
		FileWriter treewriter = new FileWriter(network_file);
		treewriter.write("parent\tchild\tdistance\n");
		while(entities.size()>0){
			//find the closest pair between the attached and available roots and the unattached entities
			String chosen_parent = "";
			String chosen_child = "";
			double min = 100000;
			for(String candidate : root_candidates.keySet()){
				for(String entity : entities){
					double distance = concept_concept_distance.get(candidate+"_"+entity);
					if((!candidate.equals(entity))&&distance < min){
						chosen_parent = candidate;
						chosen_child = entity;
						min = distance;
					}
				}
			}
			treewriter.write(chosen_parent+"\t"+chosen_child+"\t"+min+"\n");
			int child_count = root_candidates.get(chosen_parent);
			if(child_count+1==max_children){
				root_candidates.remove(chosen_parent);
			}else{
				root_candidates.put(chosen_parent, child_count+1);
			}
			entities.remove(chosen_child);
			root_candidates.put(chosen_child, 0);
		}
		treewriter.close();
	}

	public void writeReport(Set<String> targets, String genefile, String reportfile) throws IOException{
		//Set<String> genes = loadConcepts(genefile);
		Map<String, String> gene_id = loadConceptIdMap(genefile);
		FileWriter out = new FileWriter(reportfile);
		out.write("entrez_id\ttarget\ttarget_count\tgene\tgene_count\tboth\tboth_count\tpubmed_distance\n");
		for(String target : targets){
			for(String gene : gene_id.keySet()){
				String both = target+" [Title/Abstract]"+" AND "+gene+" [Title/Abstract]";
				String row = gene_id.get(gene)+"\t"+target+"\t"+concept_count.get(target)+"\t"+gene+"\t"+concept_count.get(gene)+"\t"+both+"\t"+concept_count.get(both);
				double dist = calculatePubmedDistance(concept_count.get(target), concept_count.get(gene), concept_count.get(both));
				row+= "\t"+dist;
				System.out.println(row);
				out.write(row+"\n");
			}
		}
		out.close();
	}

	public void save_search_result(String searchstring, double hits, String hitfile){
		concept_count.put(searchstring, hits);
		if(concept_q!=null){
			concept_q.remove(searchstring);
		}
		try {
			FileWriter f = new FileWriter(hitfile, true);
			f.write(searchstring+"\t"+hits+"\n");
			System.out.println(searchstring+"\t"+hits);
			f.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void distanceForConceptSets(Set<String> targets, String hitfile){
		//first get the target counts
		for(String target : targets){
			//check if we have it
			if(!concept_count.keySet().contains(target)){
				try {
					double c = getPubmedCount(target, "[Title/Abstract]");
					//if not get it
					save_search_result(target, c, hitfile);
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		//now get everything else
		for(String concept : new HashSet<String>(concept_q)){
			//check if we have it
			if(!concept_count.keySet().contains(concept)){
				try {
					double c = getPubmedCount(concept, "[Title/Abstract]");
					//if not get it
					save_search_result(concept, c, hitfile);
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		//now get the intersections
		for(String target : targets){
			for(String concept : new HashSet<String>(concept_q)){
				String both = target+" [Title/Abstract]"+" AND "+concept+" [Title/Abstract]";
				//check if we have it
				if(!concept_count.keySet().contains(both)){
					try {
						double c = getPubmedCount(both, null);
						//if not get it
						save_search_result(both, c, hitfile);
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}
	}

	public Set<String> loadConcepts(String file, int limit){
		Set<String> concepts = new HashSet<String>();
		File test = new File(file);
		if(!test.exists()){
			return concepts;
		}
		BufferedReader f;
		try {
			f = new BufferedReader(new FileReader(file));
			String line = f.readLine();
			while(line!=null){
				if(!line.startsWith("#")){
					if(line.contains("\t")){
						String[] item = line.split("\t");
						concepts.add(item[0]);
					}else{
						concepts.add(line.trim());
					}
				}
				line = f.readLine();
				if(concepts.size()>=limit){
					break;
				}
			}
			f.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return concepts;
	}

	/**
	 * Read in a two column file - col1 = search term, col2 = n hits.
	 * @param file
	 * @return
	 */
	public void loadCachedHitCounts(String file){
		concept_count = new HashMap<String, Double>(); 
		BufferedReader f;
		try {
			File test = new File(file);
			if(!test.exists()){
				FileWriter fw = new FileWriter(file);
				fw.write("");
				fw.close();
				return;
			}
			f = new BufferedReader(new FileReader(file));
			String line = f.readLine();
			while(line!=null){
				String[] item = line.split("\t");
				concept_count.put(item[0], Double.parseDouble(item[1]));
				line = f.readLine();
			}
			f.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return;
	}


	public Map<String, String> loadConceptIdMap(String file){
		Map<String, String> map = new HashMap<String, String>(); 
		BufferedReader f;
		try {
			f = new BufferedReader(new FileReader(file));
			String line = f.readLine();
			while(line!=null){
				String[] item = line.split("\t");
				map.put(item[0], item[1]);
				line = f.readLine();
			}
			f.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return map;
	}


	public void setCurrentPubmedSize(){
		//there is an axis-related bug in the response
		// as of today Oct 17, 2014
		current_pubmed_size = (double)24286891;

		//		String r = "";		
		//		// call NCBI Einfo utility
		//		EUtilsServiceStub.EInfoRequest req = new EUtilsServiceStub.EInfoRequest();	
		//		req.setTool("personal_research");
		//		req.setEmail("bgood@scripps.edu");
		//		req.setDb("pubmed"); 
		//
		//		EUtilsServiceStub.EInfoResult res;
		//		try {
		//			res = service.run_eInfo(req);
		//			// results output
		//			if(res==null){
		//				return;
		//			}
		//			DbInfoType info = res.getDbInfo();
		//			if(info!=null){
		//				String dbname = info.getDbName();
		//				current_pubmed_size = Double.parseDouble(info.getCount());
		////				String lastupdate = info.getLastUpdate();
		////				r = dbname+" last updated "+lastupdate+" now at "+n+" entries";
		//			}
		//		} catch (RemoteException e) {
		//			e.printStackTrace();
		//		}		
		return;
	}

	public double getPubmedCount(String concept1, String constraints) throws RemoteException{
		if(constraints!=null){
			concept1 += " "+constraints; //e.g. [Title/Abstract] or [MeSH Terms]		
		}
		req1.setTerm(concept1); 
		//Run ESearch 
		EUtilsServiceStub.ESearchResult res1 = service.run_eSearch(req1);
		// results output
		double c1_count = Double.parseDouble(res1.getCount());
		return c1_count;

	}

	public double getPubmedDistance(String concept1, String concept2) throws RemoteException{
		double d = 0;
		concept1 += "[Title/Abstract]";
		concept2 += "[Title/Abstract]";

		req1.setTerm(concept1); 
		//Run ESearch 
		EUtilsServiceStub.ESearchResult res1 = service.run_eSearch(req1);
		// results output
		double c1_count = Double.parseDouble(res1.getCount());
		req1.setTerm(concept2); 
		//Run ESearch 
		res1 = service.run_eSearch(req1);
		// results output
		double c2_count = Double.parseDouble(res1.getCount());

		req1.setTerm(concept2+" AND "+concept1); 
		//System.out.println(concept2+" AND "+concept1);
		//Run ESearch 
		res1 = service.run_eSearch(req1);
		// results output
		double both_count = Double.parseDouble(res1.getCount());		
		//System.out.println(c1_count+"\t"+c2_count+"\t"+both_count);

		d = calculatePubmedDistance(c1_count, c2_count, both_count);
		return d;

	}

	/**
	 * Formulated like the Google Distance
	 * See e.g. http://www.ncbi.nlm.nih.gov/pmc/articles/PMC2764279/
	 * @param count1
	 * @param count2
	 * @param intersect
	 * @return
	 */
	public double calculatePubmedDistance(double count1, double count2, double intersect){
		if(intersect==0){
			return 10000;
		}
		double d = 0;
		double M = this.current_pubmed_size;
		double bigger = count1;
		double smaller = count2;
		if(count2>count1){
			bigger = count2;
			smaller = count1;
		}
		d = (Math.log(bigger) - Math.log(intersect))/(Math.log(M) - Math.log(smaller));
		return d;
	}


	public void test() throws RemoteException{
		//PubmedDistance pdist = new PubmedDistance();
		String c1 = "brca1";  
		String c2 = "breast cancer";
		double d = getPubmedDistance(c1, c2);
		System.out.println("Distance "+c1+"\t"+c2+"\t"+d);
		c1 = "Dwarfism";  
		d = getPubmedDistance(c1, c2);
		System.out.println("Distance "+c1+"\t"+c2+"\t"+d);
		c1 = "melanoma";  
		d = getPubmedDistance(c1, c2);
		System.out.println("Distance "+c1+"\t"+c2+"\t"+d);
		c1 = "bicycle";  
		d = getPubmedDistance(c1, c2);
		System.out.println("Distance "+c1+"\t"+c2+"\t"+d);
		c1 = "HOXA9";  
		d = getPubmedDistance(c1, c2);
		System.out.println("Distance "+c1+"\t"+c2+"\t"+d);
		c1 = "JUND";  
		d = getPubmedDistance(c1, c2);
		System.out.println("Distance "+c1+"\t"+c2+"\t"+d);
	}

}
