/**
 * 
 */
package org.scripps.samesame;

import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.axis2.AxisFault;
import org.scripps.eutils.PubMed;
import org.scripps.samesame.PubmedDistance;
import org.scripps.util.FileFun;
import org.scripps.util.MapFun;

/**
 * @author bgood
 *
 */
public class Same {

	PubMed pubmed;
	//replace with a database eventually. Map of mesh term / query string and Pubmed result counts
	String global_mesh_count_file =  "data/mesh_counts.txt";
	
	public Same(String purpose, String email) throws AxisFault{
		//establish client for PubMed interactions
		pubmed = new PubMed(purpose, email); 
		//TODO add UMLS client to get semantic information
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//params
		String query = "\"endoplasmic reticulum-associated degradation\"";
		String fileroot = "data/test/";
		String purpose = "personal_research";
		String email = "bgood@scripps.edu";
		int max_terms = 50;
		//other examples
		//"ctns OR cystinosis OR cysteamine";
		//	"Ngly1 OR PNGase F";"; 
		//"ebola"; 
		String single_term_count_results = fileroot+"counts.txt";
		//"data/cystinosis/counts.txt"; 
		//"data/ngly1/counts.txt"; 
		String semantic_distance_data = fileroot+"semantic.txt";
		//"data/cystinosis/semantic_v2.txt"; 
		//"data/ngly1/semantic.txt"; 
		Same same = null;
		try {
			same = new Same(purpose, email);
			same.gatherMeshCloudData(query, single_term_count_results, semantic_distance_data);
			//now make the distance matrix and the tree-structured graph for visualization
			PubmedDistance p = new PubmedDistance();
			//opportunity for manual verification step in here
			//replace this file with a manually edited version
			String meshtermfile = semantic_distance_data;  
			//all by all distance matrix accoridng to PubmedDistance metric
			String distance_matrix_file = fileroot+"selected_terms_dist.txt";
			//this file can be imported into cytoscape as a network
			String network_file = fileroot+"selected_terms_binarytree.txt"; 
			//root should be equal to the original query.. hence it might not be a MeSH term..
			String root = query;
			p.allByAllMesh(same.global_mesh_count_file, meshtermfile, distance_matrix_file, network_file, root, max_terms); 
			
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Gather data for a mesh cloud around the input query	
	 * @param mesh_count_file  (this is the cache of all mesh term counts)
	 * @param query
	 * @param count_results
	 * @param semantic_data
	 * @throws IOException
	 */
		public void gatherMeshCloudData(String query, String single_term_count_results, String semantic_data) throws IOException{
			int q_count = pubmed.getPmidCountForTerm(query);
			Map<String,Set<String>> pmid_indexes = pubmed.getAllIndexTerms(pubmed.searchPubmed(query, 500, null, 0));
			Map<String, Integer> term_query_count = new HashMap<String, Integer>();
			//now get the rest
			for(String pmid_type : pmid_indexes.keySet()){
				if(pmid_indexes.get(pmid_type)!=null){
					//System.out.println(pmid_type+"\t"+pmid_indexes.get(pmid_type));
					if(pmid_type.contains("mesh")){
						for(String term : pmid_indexes.get(pmid_type)){
							Integer c = term_query_count.get(term);
							if(c == null){c = 0;}
							c++;
							term_query_count.put(term, c);
						}
					}
				}
			}
			Map<String, Float> term_freq = MapFun.convertCountsToPercentages(term_query_count);
			//write out result counts
			FileWriter out = new FileWriter(single_term_count_results);
			for(String term : term_freq.keySet()){
				out.write(term+"\t"+term_query_count.get(term)+"\n");
			}
			out.close();
			//update mesh to pmid count table.
			buildGlobalMeshCountTable(single_term_count_results, global_mesh_count_file);
			Map<String, Integer> term_all_count = FileFun.readMap(global_mesh_count_file, 10000000);
			Map<String, Float> all_term_freq = MapFun.convertCountsToPercentages(term_all_count);

			PubmedDistance pubdistance = new PubmedDistance();
			Map<String, Double> term_pubdist = new HashMap<String, Double>();
			Map<String, String> term_label = new HashMap<String, String>();
			for(String term : term_freq.keySet()){
				double pubdist = pubdistance.calculatePubmedDistance(q_count, term_all_count.get(term), term_query_count.get(term));
				String cytoscape_label = term+" (all "+term_all_count.get(term)+") (co-occur "+term_query_count.get(term)+")";
				term_pubdist.put(term, pubdist);
				term_label.put(term, cytoscape_label);
			}
			//sort
			List<String> sorted_terms = MapFun.sortMapByValue(term_pubdist);
			out = new FileWriter(semantic_data);
			out.write("#term\tcytoscape_label\tpubdist\tquery_freq\tquery_count\tall_pubmed_freq\tall_pubmed_count\n");
			for(String term : sorted_terms){
				//zero distance means something went wrong somewhere..
				if(term_pubdist.get(term)!=0){
					out.write(term+"\t"+term_label.get(term)+"\t"+term_pubdist.get(term)+"\t"+term_freq.get(term)+"\t"+term_query_count.get(term)+"\t"+all_term_freq.get(term)+"\t"+term_all_count.get(term)+"\n");
				}
			}
			out.close();			
			//write file with semantic data
			//TODO, hook up to UMLS web service.  This one ran with a local copy of the umls as mysql.
//			out = new FileWriter(semantic_data);
//			out.write("#term\tcytoscape_label\tpubdist\tquery_freq\tquery_count\tall_pubmed_freq\tall_pubmed_count\tgroups\ttypes\n");
//			PubmedDistance pubdistance = new PubmedDistance();
//			UmlsDb umls = new UmlsDb();
//			for(String term : term_freq.keySet()){
//				Set<String> types = umls.getSemanticTypesForMeshAtom(term);
//				Set<String> groups = new HashSet<String>();
//				if(types!=null){
//					for(String type : types){
//						groups.add(umls.getGroupForStypeName(type));
//					}
//				}
//				double pubdist = pubdistance.calculatePubmedDistance(q_count, term_all_count.get(term), term_query_count.get(term));
//				String cytoscape_label = term+" (all "+term_all_count.get(term)+") (co-occur "+term_query_count.get(term)+")";
//				out.write(term+"\t"+cytoscape_label+"\t"+pubdist+"\t"+term_freq.get(term)+"\t"+term_query_count.get(term)+"\t"+all_term_freq.get(term)+"\t"+term_all_count.get(term)+"\t"+groups+"\t"+types+"\n");
//			}
//			out.close();
		}
		
		
		
		/*
		 * Read in a list of mesh terms of interest
		 * Write out the article counts for each term
		 */
		public void buildGlobalMeshCountTable(String input, String output) throws IOException{
			Map<String, Integer> term_count = FileFun.readMap(output, 10000000);
			Set<String> terms = FileFun.readOneColFromFile(input, 0, 10000);
			terms.remove("term");
			System.out.println("Looking for "+terms.size()+" mesh counts, already have in cache: "+term_count.keySet().size());
			terms.removeAll(term_count.keySet());
			System.out.println("Gathering "+terms.size()+" new mesh counts");		
			int n = 0;
			for(String term : terms){
				long start = System.currentTimeMillis()/1000;
				n++;
				int c = pubmed.getPmidCountForMeshTerm(term);
				FileWriter f = new FileWriter(output, true);
				f.write(term+"\t"+c+"\n");
				f.close();
				System.out.println(n+"\t"+term+"\t"+c+"\t"+(System.currentTimeMillis()/1000-start));
			}
		}
}
